/**
 * Created on:		15 Sep 2010
 * @author:			Libor Kadlec
 *
 * Copyright (c) Changing Worlds Ltd. 2010.
*/
package com.amdocs.ycsb;

import java.util.HashMap;
import java.util.Properties;
import java.util.Random;
import java.util.Set;
import java.util.Vector;

import javax.jms.BytesMessage;
import javax.jms.Connection;
import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;

import com.yahoo.ycsb.DB;
import com.yahoo.ycsb.DBException;

public abstract class JMSClient extends DB {
    
    private static final String DELIVERY_MODE_NON_PERSISTENT = "NON_PERSISTENT";
    
    private int deliveryMode;
    private Queue queue;
    private Session session;
    private MessageProducer producer;
    private MessageConsumer consumer;
    private boolean messageListener = false;
    private int messageLength = 1024;
    private BytesMessage message = null;
    private int consumersPerProducer = 1;
    
    protected abstract Connection getConnection() throws JMSException;
    protected abstract Queue createQueue(String name);
    
    public void init() throws DBException {
        try {
            Properties p = getProperties();
            readProperties(p);
            Connection con = getConnection();
            this.session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
            this.queue = createQueue(p.getProperty("queue.name"));
            this.consumer = this.session.createConsumer(this.queue);
            if (this.messageListener) {
                MessageListenerImpl messageListener = new MessageListenerImpl();
                this.consumer.setMessageListener(messageListener);
                for (int i = 0; i < this.consumersPerProducer - 1; i++) {
                    con.createSession(false, Session.AUTO_ACKNOWLEDGE).createConsumer(this.queue).setMessageListener(messageListener);
                }
            }
            this.producer = this.session.createProducer(this.queue);
            this.producer.setDeliveryMode(this.deliveryMode);
            this.message = this.session.createBytesMessage();
            this.message.writeBytes(randomByteArray(this.messageLength));
        } catch (Exception e) {
            throw new DBException(e);
        }
    }
    
    private void readProperties(Properties p) {
        String deliveryModeStr = p.getProperty("messageProducer.deliveryMode");
        if (deliveryModeStr != null && deliveryModeStr.trim().equals(DELIVERY_MODE_NON_PERSISTENT)) {
            this.deliveryMode = DeliveryMode.NON_PERSISTENT;
        } else {
            this.deliveryMode = DeliveryMode.PERSISTENT;
        }
        this.messageListener = Boolean.parseBoolean(p.getProperty("messageListener", "false"));
        this.messageLength = Integer.parseInt(p.getProperty("messageLength", "1024"));
        this.consumersPerProducer = Integer.parseInt(p.getProperty("consumersPerProducer", "1"));
    }
    
    @Override
    public int read(String table, String key, Set<String> fields, HashMap<String, String> result) {
        try {
            this.consumer.receive(800);
            return 0;
        } catch (JMSException e) {
            e.printStackTrace();
            return 1;
        }
    }

    @Override
    public int insert(String table, String key, HashMap<String, String> values) {
        try {
            producer.send(this.message);
        } catch (JMSException e) {
            e.printStackTrace();
            return 1;
        }
        return 0;
    }

    @Override
    public int scan(String table, String startkey, int recordcount, Set<String> fields, Vector<HashMap<String, String>> result) {
        return 0;
    }

    @Override
    public int update(String table, String key, HashMap<String, String> values) {
        return 0;
    }

    @Override
    public int delete(String table, String key) {
        return 0;
    }
    
    public void cleanup() throws DBException {
    }
    
    private byte[] randomByteArray(final int length) {
        byte[] bytes = new byte[length];
        Random random = new Random();
        for (int i = 0; i < length; i++) {
            bytes[i] = Integer.valueOf(random.nextInt()).byteValue();
        }
        return bytes;
    }
}
